import os
import re

from mediasort.classify.classification import Classification, MEDIA_TYPES


SEASON_NAME_REGEX = re.compile(r'(.*)(S\d\d?E\d\d?)', re.IGNORECASE)
WHOLE_SEASON_REGEX = re.compile(r'(.*)(Season.?\d+)', re.IGNORECASE)

def detect_tv(path):
    f = os.path.basename(path)

    match = SEASON_NAME_REGEX.search(f)
    if match:
        return Classification(path, MEDIA_TYPES.tv, 6, name=match.group(1))

    match = WHOLE_SEASON_REGEX.search(f)
    if match:
        return Classification(path, MEDIA_TYPES.tv, 4, name=match.group(1))

    return Classification.none(path)




import logging
logger = logging.getLogger(__name__)

import os
import shutil

import click

from fuzzywuzzy import process

from mediasort.act import Action
from mediasort.lib import as_list
from mediasort.lib import validate


class CopyTo(Action):
    __key__ = "copy_to"

    def validate_options(self):
        self.dir_must_exist(self.destination(), 'destination')

    def destination(self):
        return self.options.get('destination')

    def perform(self, dry_run):
        path = self.options['target'].path
        self.copy_file_or_dir(path, dry_run=dry_run)

    def copy_file_or_dir(self, path, dry_run):
        logger.info("copying '%s' to '%s'" % (path, self.destination()))
        if os.path.isfile(path):
            action = self.copy_file
        elif os.path.isdir(path):
            action = self.copy_dir
        else:
            raise click.ClickException("Couldn't find path: %s" % path)

        if not dry_run:
            action(path, self.destination())


    def copy_file(self, src, dst_root):
        shutil.copy(src, dst_root)

    def copy_dir(self, src, dst_root):
        dst = os.path.join(dst_root, os.path.basename(src))
        shutil.copytree(src, dst)
        mode = self.options.get('permissions')
        if mode:
            os.chmod(dst, mode)


class CopyToSubdir(CopyTo):
    __key__ = 'copy_to_subdir'

    def validate_options(self):
        self.dir_must_exist(self.options.get('destination'), 'destination')
        self.validate_subdir()
        self.ensure_destination_exists()

    def ensure_destination_exists(self):
        path = self.destination()
        if not os.path.exists(path):
            os.makedirs(path)
            mode = self.options.get('permissions')
            if mode:
                os.chmod(path, mode)

    def validate_subdir(self):
        if 'subdir' not in self.options:
            raise click.ClickException("copy_to_subdir needs a subdir")

    def subdir():
        return self.options.get('subdir')

    def destination(self):
        return os.path.join(*filter(None, [
            self.options.get('destination'),
            self.subdir(),
        ]))

class CopyToMatchingSubdir(CopyToSubdir):
    __key__ = 'copy_to_matching_subdir'

    def validate_subdir(self):
        if 'subdir' in self.options:
            raise click.ClickException(
                "can't pass a subdir to copy_to_matching_subdir"
            )

    def subdir(self):
        if 'chosen_subdir' not in self.options:
            self.choose_subdir()
        return self.options['chosen_subdir']

    def choose_subdir(self):
        root = self.options.get('destination')
        haystack = [d for d in os.listdir(root)
                    if os.path.isdir(os.path.join(root, d))]

        name = os.path.basename(self.options['target'].path)
        choice = process.extractOne(name, haystack)

        if choice[1] >= self.options.get('score_cutoff', 90):
            self.options['chosen_subdir'] = choice[0]
        else:
            self.options['chosen_subdir'] = self.options['target'].name


class CopyContentsTo(CopyToSubdir):
    __key__ = 'copy_contents_to'

    def subdir(self):
        if self.options.get('preserve_dir', False):
            return os.path.basename(self.options['target'].path)

        return None

    def validate_subdir(self): pass

    def perform(self, dry_run):
        path = self.options['target'].path
        contents = [os.path.join(path, f) for f in os.listdir(path)]
        only = as_list(self.options.get('only', []))
        exclude = as_list(self.options.get('exclude', []))

        if only:
            logger.debug("Only: %s" % only)
        if exclude:
            logger.debug("Exclude: %s" % exclude)

        for f in contents:
            ext = os.path.splitext(f)[1]
            if ext.startswith('.'):
                ext = ext[1:]

            if ((only and ext in only) or
              (not only and exclude and ext not in exclude) or
              (not only and not exclude)):
                self.copy_file_or_dir(f, dry_run=dry_run)

            else:
                logger.debug("%s did not pass extension filter" % f)




from mediasort import setup_logging

from mediasort.classify import Classifier
from mediasort.classify.classification import Classification

from mediasort.act import Action

import logging
logger = logging.getLogger(__name__)

import click
import yaml

config_option = click.option('--config', '-c',
                             type=click.File(),
                             help='path to config.yml')
dry_run_option = click.option('--dry-run', '-d',
                              help='don\'t make any filesystem changes',
                              is_flag=True,
                              default=False)
path_option = click.argument('path')

@click.group()
@config_option
@click.pass_context
def cli(ctx, config):
    conf = yaml.load(config)
    setup_logging(conf)

    ctx.obj = dict(
        config=conf
    )

@cli.command()
@path_option
def test(ctx, path):
    pass

@cli.command()
@path_option
def validate(ctx):
    pass

@cli.command()
@dry_run_option
@path_option
@click.pass_context
def classify(ctx, dry_run, path):
    config = ctx.obj['config']

    if not config:
        raise click.ClickException("You must provide a config")

    classifier = Classifier(config)

    if path.endswith('/'):
        path = path[:-1]

    rv = classifier.classify(path)

    logger.info("%s: %s" % (path, rv))

    for action_set in classifier.actions_for(rv):
        for action in Action.from_config(action_set, rv):
            action.perform(dry_run=dry_run)



from collections import namedtuple
def enum(*keys, **kw):
    return namedtuple('Enum', keys)(*keys)

def as_list(obj, types=(list)):
    if not isinstance(obj, types):
        obj = [obj]
    return obj




import os
import re

from mediasort.classify.omdb import Omdb
from mediasort.classify.classification import Classification, MEDIA_TYPES

from mediasort.classify.tv import SEASON_NAME_REGEX

import logging
logger = logging.getLogger(__name__)

TITLE_AND_YEAR_REGEX = re.compile(r'^(.*)[.-_ ](\d{4})')
HD_YEAR_RES_REGEX = re.compile(
    r'^(.*)[.-_ ](\d{4})[.-_ ]((?:480|720|1080)(?:p|i))',
    re.IGNORECASE
)

omdb = Omdb()

def detect_movie(path):
    score = 0

    f = os.path.basename(path)

    if SEASON_NAME_REGEX.search(f):
        return Classification.none(path)

    hd_match = HD_YEAR_RES_REGEX.search(f)
    title_match = TITLE_AND_YEAR_REGEX.search(f)

    if hd_match:
        resolution = hd_match.group(3)
        parts = re.split('[ -._]', hd_match.group(1))
        title = ' '.join(filter(None, parts))
        year = hd_match.group(2)
        score = 8

    elif title_match:
        parts = re.split('[-_. ]', title_match.group(1))
        title = ' '.join(filter(None, parts))
        year = title_match.group(2)
        score = 6
    else:
        return Classification.none(path)

    logger.debug(title)
    logger.debug(year)

    rv = omdb.api(title=title, year=year)
    if rv:
        logger.debug('Response: %s' % rv.get("Response"))
        logger.debug('Type: %s' % rv.get("Type"))
        if rv.get("Response") == "True" and rv.get("Type") == "movie":
            return Classification(path, MEDIA_TYPES.movie, score, name=title)

    return Classification.none(path)



import os

from mediasort.classify.classification import Classification, MEDIA_TYPES

def detect_music(path, types=None):
    count = len(types)
    if count > 100:
        return Classification.none(path)

    music_types = [t for t in types if 'audio' in t]
    score = int(round(float(len(music_types)) * 10 / count))

    mtype = None
    if any(['flac' in t for t in music_types]):
        mtype = MEDIA_TYPES.lossless_music
    else:
        mtype = MEDIA_TYPES.music

    if mtype:
        return Classification(path, mtype, score, name=os.path.basename(path))

    return Classification.none(path)




import operator
import os
import mimetypes

from mediasort.classify.omdb import Omdb
from mediasort.classify.imdb import (
    find_nfos,
    extract_imdb_id,
)

from mediasort.classify.tv import detect_tv
from mediasort.classify.movies import detect_movie
from mediasort.classify.music import detect_music


from mediasort.classify.classification import (
    MEDIA_TYPES,
    Classification,
)

import logging
logger = logging.getLogger(__name__)

class Classifier(object):

    def __init__(self, config):
        self.omdb = Omdb()
        self.config = config

    def get_types(self, path):
        types = []
        if os.path.isdir(path):
            for folder, _, files in os.walk(path):
                for filename in files:
                    types.append(mimetypes.guess_type(filename)[0])

        else:
            types.append(mimetypes.guess_type(path)[0])

        return filter(None, list(set(types)))

    def classify(self, path):
        types = self.get_types(path)

        classifications = [
            self.classify_by_nfo(path),
            Classification.none(path),
        ]

        if any(['video' in t for t in types]):
            classifications.extend([
                detect_tv(path),
                detect_movie(path),
            ])

        if any(['audio' in t for t in types]):
            classifications.extend([
                detect_music(path, types),
            ])

        return max(classifications)

    def actions_for(self, classification):
        actions = self.config.get('actions', [])
        if not actions:
            logger.warn("No actions registered in config")
            return []

        return [a for a in actions
                if a['media_type'] == classification.media_type and
                a['confidence'] <= classification.score]


    def classify_by_nfo(self, path):

        if os.path.isdir(path):
            nfos = find_nfos(path)
            ids = [extract_imdb_id(nfo) for nfo in nfos]
            omdb_responses = filter(None, [self.omdb.api(imdb=i) for i in ids])
            omdb_types = map(operator.itemgetter('Type'), omdb_responses)

            if 'movie' in omdb_types:
                return Classification(path, MEDIA_TYPES.movie, 10)

            elif set(['episode', 'series']).intersection(omdb_types):
                return Classification(path, MEDIA_TYPES.tv, 10)

        return Classification.none(path)



import re
import os

IMDB_URL_REGEX = re.compile(r'^http://www.imdb.com/title/(tt[0-9]+)/$',
                            re.IGNORECASE)
IMDB_ID_REGEX = re.compile(r'(tt\d{7})')


def find_nfos(directory):
    return [os.path.join(directory, f) for f in os.listdir(directory)
            if f.endswith('.nfo')]

def extract_imdb_id(nfo_file):
    with open(nfo_file, 'r') as f:
        data = f.read()
        match = IMDB_URL_REGEX.search(data)
        if match:
            return match.group(1)

        match = IMDB_ID_REGEX.search(data)
        if match:
            return match.group(1)




import requests

import logging
logger = logging.getLogger(__name__)

class Omdb(object):

    def api(self, title=None, imdb=None, year=None):
        if not (title or imdb):
            return None

        params = dict(t=title, i=imdb, y=year)
        resp = requests.get("http://www.omdbapi.com", params=params)
        try:
            answer = resp.json()
            response_val = answer.get('Response')
            if response_val == 'True':
                return answer

            elif response_val == 'False':
                logger.error('[OMDB] Error: %s (from %s)' % (
                    answer.get('Error'), params
                ))
            else:
                logger.error('[OMDB] Unknown: %s (from %s)' % (answer, params))

            return None

        except:
            logger.error("[OMDB] Request error:")
            for line in r.iter_lines():
                logger.error(line)

        return None





from mediasort.lib import enum
MEDIA_TYPES = enum(
    'tv',
    'movie',
    'music',
    'lossless_music',
    'other'
)

import re
SPACERS = re.compile(r'[ -._]')


class Classification(object):
    def __init__(self, path, mtype, score, name=None):
        self.path = path
        self.media_type = mtype
        self.score = score
        self.name = self.normalize_name(name)

    @classmethod
    def normalize_name(cls, name):
        if not name:
            return name
        return SPACERS.sub(' ', name).strip()

    @classmethod
    def none(cls, path):
        return cls(path=path, mtype=MEDIA_TYPES.other, score=0)

    def __eq__(self, other):
        try:
            return all([self.media_type == other.media_type,
                        self.score == other.score])
        except:
            return False

    def __lt__(self, other):
        return self.score < other.score

    def __str__(self):
        rv = "%s(%s)" % (self.media_type, self.score)
        if self.name:
            rv += " %s" % self.name
        return rv

    def __repr__(self):
        return str(self)




#!/usr/bin/env python

from setuptools import setup, find_packages
version = "1.2"

long_description = """
Perform an action based on media type
"""

setup(
    name='mediasort',
    version=version,
    description='Media type detection',
    license='BSD',

    long_description=long_description,
    author='Andrew Roberts',
    author_email='adroberts@gmail.com',
    url='https://github.com/aroberts',

    packages=find_packages(),

    install_requires=['requests', 'PyYAML', 'click', 'fuzzywuzzy'],

    entry_points={
        'console_scripts': [
            'mediasort = mediasort.cli:cli',
        ]
    },

    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
    ],
)



import logging
logger = logging.getLogger(__name__)

handler = logging.StreamHandler()
formatter = logging.Formatter(
    '%(asctime)s %(levelname)s %(message)s',
    "%Y-%m-%d %H:%M:%S"
)
handler.setFormatter(formatter)
logger.addHandler(handler)
logger.setLevel(logging.INFO)

def setup_logging(config):
    if 'log_path' in config:
        handler = logging.FileHandler(config['log_path'])
        handler.setFormatter(formatter)
        logger.addHandler(handler)

    if 'log_level' in config:
        logger.setLevel(getattr(logging, config['log_level'].upper()))



import click
import os

class Validatable(object):
    def dir_must_exist(self, path, key):
        msg = "%s requires '%s' to be a directory that exists (%s)" % (
            self.__key__, key, path
        )

        if path == None or not os.path.isdir(path):
            raise click.ClickException(msg)




import logging
logger = logging.getLogger(__name__)

import click

from mediasort.lib.validate import Validatable

registry = {}

NO_KEY = object()

class ActionMetaclass(type):
    def __new__(cls, name, bases, attrs):
        cls = type.__new__(cls, name, bases, attrs)
        key = attrs.get('__key__', NO_KEY)
        if key == NO_KEY:
            raise Exception("Need to supply __key__ with Action declaration")

        if key in registry:
            raise Exception("'%s' already taken: '%s'" % (key, registry[key]))

        registry[key] = cls
        return cls

class Action(Validatable):
    __metaclass__ = ActionMetaclass
    __key__ = None

    def __init__(self, options):
        self.options = options

    @classmethod
    def from_config(cls, action_hash, classification):

        definitions = action_hash['perform']
        if isinstance(definitions, dict):
            definitions = [definitions]

        try:
            actions = [
                registry[action](options)
                for d in definitions
                for action, options in d.items()
            ]

            for action in actions:
                action.options['target'] = classification
                action.validate_options()

            return actions
        except KeyError, e:
            msg = "Available actions: %s" % registry.keys()
            logger.error(e.message)
            logger.error(msg)
            raise click.ClickException(msg)

    def perform(**kwargs):
        raise NotImplementedError()

    def validate_options(self):
        pass


from mediasort.act.copy_to import CopyTo
from mediasort.act.copy_to import CopyContentsTo
